# !/usr/bin/env Python3
# -*- coding:utf-8 -*-
# Team : HiLab
# Author：Yu Tao
# Date ：2022/6/20 20:15
# Tool ：PyCharm
# Email : yuao_11260103@163.com
# Project : msDAWG
from sklearn.metrics import confusion_matrix, roc_curve, auc
import xgboost


def train_test_model(model, data_dict):
    """
    train on model.return result on model of test

    :param model: xgboost..etc
    :param data_dict: data dict
    :return: model, y_pred, prob, cfu_matrix
    """
    model.fit(data_dict['X_train'], data_dict['y_train'])
    y_pred = model.predict(data_dict['X_test'])
    prob = model.predict_proba(data_dict['X_test'])
    cfu_matrix = confusion_matrix(data_dict['y_test'], y_pred).ravel()
    # tn, fp, fn, tp = cfu_matrix

    return model, y_pred, prob, cfu_matrix


def cal_auc(y_test, y_prob):
    """
    calculate auc

    :param y_test:y_true
    :param y_prob: model.predict_proba
    :return:result_auc -> float
    """
    a, b, c = roc_curve(y_test, y_prob[:, 1])
    result_auc = auc(a, b)
    return result_auc


def cal_score(cfu_matrix):
    """
    calculate precision,recall,accuracy
    :param cfu_matrix: confusion_matrix
    :return:precision, recall, accuracy
    """
    tn, fp, fn, tp = cfu_matrix
    precision = tp / (tp + fp)
    recall = tp / (tp + fn)
    accuracy = (tp + tn) / sum(cfu_matrix)
    # print("precision = {precision}\nrecall = {recall}\naccuracy = {accuracy}\n".format(
    #     precision=precision, recall=recall, accuracy=accuracy))
    return precision, recall, accuracy


def all_evaluate(X_train, X_test, y_train, y_test):
    """
    put in data,return model result

    :param X_train: arr
    :param X_test: arr
    :param y_train: arr
    :param y_test: arr
    :return:precision, recall, accuracy, aucc ->arr, arr, arr, arr
    """
    xgb = xgboost.XGBClassifier(n_estimators=500, subsample=1)
    data_dict = {'X_train': X_train, 'X_test': X_test, 'y_train': y_train, 'y_test': y_test}
    _, _, prob, cfu_matrix = train_test_model(xgb, data_dict)
    precision, recall, accuracy = cal_score(cfu_matrix)
    aucc = cal_auc(data_dict['y_test'], prob)
    print("{precision}\n{recall}\n{accuracy}\n{aucc}".format(
        precision=precision, recall=recall, accuracy=accuracy, aucc=aucc))
    return precision, recall, accuracy, aucc


